@@ -15,7 +15,7 @@
  *
  * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2002
+ * Portions created by the Initial Developer are Copyright © 2002
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
@@ -36,25 +36,29 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: EphyHeaderSniffer.h,v 1.14 2004/11/13 19:53:38 chpe Exp $
+ * $Id$
  */
 
+#ifndef EPHY_HEADER_SNIFFER_H
+#define EPHY_HEADER_SNIFFER_H
+
+class nsIDOMDocument;
+class nsIFile;
+class nsIInputStream;
+class nsILocalFile;
+class nsIURI;
+class nsIWebBrowserPersist;
+
+#include <nsCOMPtr.h>
+#include <nsIAuthPrompt.h>
+#include <nsIInterfaceRequestor.h>
+#include <nsIWebProgressListener.h>
+
 #include "mozilla-embed-persist.h"
 #include "ephy-embed-single.h"
 
-#include <nsIWebProgressListener.h>
-#include <nsIWebBrowserPersist.h>
-#include <nsIURI.h>
-#include <nsILocalFile.h>
-#include <nsIInputStream.h>
-#include <nsIDOMDocument.h>
-#include <nsIAuthPrompt.h>
-#include <nsIPromptService.h>
-#define MOZILLA_STRICT_API
-#include <nsEmbedString.h>
-#undef MOZILLA_STRICT_API
-
 class EphyHeaderSniffer : public nsIWebProgressListener,
+			  public nsIInterfaceRequestor,
 			  public nsIAuthPrompt
 {
 public:
@@ -66,6 +70,7 @@ public:
 	NS_DECL_ISUPPORTS
 	NS_DECL_NSIWEBPROGRESSLISTENER
 	NS_FORWARD_SAFE_NSIAUTHPROMPT(mAuthPrompt)
+	NS_DECL_NSIINTERFACEREQUESTOR
 
 	nsresult InitiateDownload (nsILocalFile *aDestFile);
 
@@ -82,8 +87,9 @@ private:
 	nsCOMPtr<nsIURI>           mOriginalURI;
 	nsCOMPtr<nsIDOMDocument>   mDocument;
 	nsCOMPtr<nsIInputStream>   mPostData;
-	nsEmbedCString             mContentType;
-	nsEmbedCString             mContentDisposition;
+	nsCString             mContentType;
+	nsCString             mContentDisposition;
 	nsCOMPtr<nsIAuthPrompt>    mAuthPrompt;
 };
 
+#endif /* !EPHY_HEADER_SNIFFER_H */
