@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-location-entry.c 8174 2008-03-31 00:03:59Z chpe $
+ *  $Id$
  */
 
 #include "config.h"
@@ -36,24 +36,7 @@
 
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtktoolbar.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkentrycompletion.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkcelllayout.h>
-#include <gtk/gtktreemodelsort.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkeventbox.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkimagemenuitem.h>
-#include <gtk/gtkseparatormenuitem.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkclipboard.h>
+#include <gtk/gtk.h>
 
 #include <string.h>
 
@@ -71,6 +54,7 @@ struct _EphyLocationEntryPrivate
 	GdkColor secure_fg_colour;
 
 	GtkCellRenderer *extracell;
+	GRegex *regex;
 
 	char *before_completion;
 	char *saved_text;
@@ -79,9 +63,9 @@ struct _EphyLocationEntryPrivate
 	guint action_col;
 	guint keywords_col;
 	guint relevance_col;
+	guint url_col;
 	guint extra_col;
 	guint favicon_col;
-	guint url_col;
 
 	guint hash;
 
@@ -231,6 +215,12 @@ ephy_location_entry_finalize (GObject *object)
 	{
 		g_object_unref (priv->favicon);
 	}
+	
+	if (priv->regex)
+	{
+		g_regex_unref (priv->regex);
+		priv->regex = NULL;
+	}
 
 	parent_class->finalize (object);
 }
@@ -345,6 +335,8 @@ editable_changed_cb (GtkEditable *editable,
 		     EphyLocationEntry *entry)
 {
 	EphyLocationEntryPrivate *priv = entry->priv;
+	const char *text;
+	char *pattern;
 
 	update_address_state (entry);
 
@@ -355,6 +347,25 @@ editable_changed_cb (GtkEditable *editable,
 		priv->user_changed = TRUE;
 		priv->can_redo = FALSE;
 	}	
+	
+	if (priv->regex)
+	{
+		g_regex_unref (priv->regex);
+		priv->regex = NULL;
+	}
+	
+	text = gtk_entry_get_text (GTK_ENTRY (editable));
+
+	if (g_str_has_prefix (text, "re:"))
+		pattern = g_strdup (text+3);
+	else
+		pattern = g_regex_escape_string (text, -1);
+
+	priv->regex = g_regex_new (pattern, 
+				G_REGEX_CASELESS | G_REGEX_OPTIMIZE, 
+				G_REGEX_MATCH_NOTEMPTY, NULL);
+
+	g_free (pattern);
 
 	update_favicon (entry);
 
@@ -516,7 +527,7 @@ entry_drag_motion_cb (GtkWidget        *widget,
 	{
 		g_signal_stop_emission_by_name (widget, "drag_motion");
 	}
-    
+
 	return FALSE;
 }
 
@@ -925,7 +936,8 @@ ephy_location_entry_init (EphyLocationEntry *le)
 	p->user_changed = FALSE;
 	p->block_update = FALSE;
 	p->saved_text = NULL;
-
+	p->regex = NULL;
+	
 	ephy_location_entry_construct_contents (le);
 
 	gtk_tool_item_set_expand (GTK_TOOL_ITEM (le), TRUE);
@@ -937,23 +949,6 @@ ephy_location_entry_new (void)
 	return GTK_WIDGET (g_object_new (EPHY_TYPE_LOCATION_ENTRY, NULL));
 }
 
-static gint
-sort_func (GtkTreeModel *model,
-	   GtkTreeIter *a,
-	   GtkTreeIter *b,
-	   gpointer data)
-{
-	gint valuea, valueb;
-	EphyLocationEntry *le = EPHY_LOCATION_ENTRY (data);
-
-	gtk_tree_model_get (model, a,
-			    le->priv->relevance_col, &valuea, -1);
-	gtk_tree_model_get (model, b,
-			    le->priv->relevance_col, &valueb, -1);
-
-	return valueb - valuea;
-}
-
 static gboolean
 cursor_on_match_cb  (GtkEntryCompletion *completion,
 		     GtkTreeModel *model,
@@ -966,8 +961,8 @@ cursor_on_match_cb  (GtkEntryCompletion *completion,
 	gtk_tree_model_get (model, iter,
 			    le->priv->url_col,
 			    &item, -1);
-	
 	entry = gtk_entry_completion_get_entry (completion);
+
 	gtk_entry_set_text (GTK_ENTRY (entry), item);
 	gtk_editable_set_position (GTK_EDITABLE (entry), -1);
 
@@ -986,8 +981,11 @@ extracell_data_func (GtkCellLayout *cell_layout,
 	char *cdata;
 	GValue visible = { 0, };
 	GValue text = { 0, };
-	
-	gtk_tree_model_get (tree_model, iter, GPOINTER_TO_UINT(data), &cdata, -1);
+	EphyLocationEntryPrivate *priv;
+
+	priv = EPHY_LOCATION_ENTRY (data)->priv;
+
+	gtk_tree_model_get (tree_model, iter, priv->extra_col, &cdata, -1);
 
 	g_value_init (&text, G_TYPE_STRING);
 	g_value_init (&visible, G_TYPE_BOOLEAN);
@@ -1003,15 +1001,16 @@ extracell_data_func (GtkCellLayout *cell_layout,
 }
 
 void
-ephy_location_entry_set_completion_func (EphyLocationEntry *le, 
-					GtkEntryCompletionMatchFunc completion_func,
-					gpointer user_data)
+ephy_location_entry_set_match_func (EphyLocationEntry *le, 
+				GtkEntryCompletionMatchFunc match_func,
+				gpointer user_data,
+				GDestroyNotify notify)
 {
 	EphyLocationEntryPrivate *priv = le->priv;
 	GtkEntryCompletion *completion;
 	
 	completion = gtk_entry_get_completion (GTK_ENTRY (priv->icon_entry->entry));
-	gtk_entry_completion_set_match_func (completion, completion_func, user_data, NULL);
+	gtk_entry_completion_set_match_func (completion, match_func, user_data, notify);
 }
 
 void
@@ -1021,9 +1020,9 @@ ephy_location_entry_set_completion (EphyLocationEntry *le,
 				    guint action_col,
 				    guint keywords_col,
 				    guint relevance_col,
+				    guint url_col,
 				    guint extra_col,
-				    guint favicon_col,
-				    guint url_col)
+				    guint favicon_col)
 {
 	EphyLocationEntryPrivate *priv = le->priv;
 	GtkTreeModel *sort_model;
@@ -1034,15 +1033,17 @@ ephy_location_entry_set_completion (EphyLocationEntry *le,
 	le->priv->action_col = action_col;
 	le->priv->keywords_col = keywords_col;
 	le->priv->relevance_col = relevance_col;
+	le->priv->url_col = url_col;
 	le->priv->extra_col = extra_col;
 	le->priv->favicon_col = favicon_col;
-	le->priv->url_col = url_col;
 
 	sort_model = gtk_tree_model_sort_new_with_model (model);
+
 	g_object_unref (model);
-	gtk_tree_sortable_set_default_sort_func
-		(GTK_TREE_SORTABLE (sort_model),
-		 sort_func, le, NULL);
+	gtk_tree_sortable_set_sort_column_id 
+			(GTK_TREE_SORTABLE (sort_model),
+			 le->priv->relevance_col,
+			 GTK_SORT_DESCENDING);
 
 	completion = gtk_entry_completion_new ();
 	gtk_entry_completion_set_model (completion, sort_model);
@@ -1081,7 +1082,7 @@ ephy_location_entry_set_completion (EphyLocationEntry *le,
 	
 	gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (completion),
 					le->priv->extracell, extracell_data_func, 
-					GUINT_TO_POINTER (extra_col),
+					le,
 					NULL);
 
 	g_object_set (completion, "inline-selection", TRUE, NULL);
@@ -1327,3 +1328,11 @@ ephy_location_entry_set_lock_tooltip (EphyLocationEntry *entry,
 
 	gtk_widget_set_tooltip_text (priv->lock_ebox, tooltip);
 }
+
+GRegex *
+ephy_location_entry_get_regex (EphyLocationEntry *entry)
+{
+	EphyLocationEntryPrivate *priv = entry->priv;
+	
+	return priv->regex;
+}
