@@ -15,13 +15,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-open-tabs-action.h 6952 2007-03-11 19:42:02Z chpe $
+ *  $Id$
  */
 
 #ifndef EPHY_OPEN_TABS_ACTION_H
 #define EPHY_OPEN_TABS_ACTION_H
 
-#include <gtk/gtkactiongroup.h>
+#include <gtk/gtk.h>
 
 #include "ephy-node.h"
 
