@@ -37,21 +37,7 @@
 #include "ephy-favicon-cache.h"
 #include "ephy-history.h"
 
-#include <gtk/gtklabel.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkhseparator.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
 #include <string.h>
@@ -561,6 +547,9 @@ pdm_dialog_remove_button_clicked_cb (GtkWidget *button,
 				     PdmActionInfo *action)
 {
 	pdm_cmd_delete_selection (action);
+
+	/* Restore the focus to the button */
+	gtk_widget_grab_focus (button);
 }
 
 static void
