@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: window-commands.c 7945 2008-02-15 01:10:36Z cosimoc $
+ *  $Id$
  */
 
 #include "config.h"
@@ -56,12 +56,7 @@
 #include <string.h>
 #include <glib.h>
 #include <gio/gio.h>
-#include <gtk/gtkaboutdialog.h>
-#include <gtk/gtkeditable.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkicontheme.h>
-#include <gtk/gtktoggleaction.h>
-#include <gtk/gtkprintoperation.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
 static void
@@ -726,22 +721,13 @@ window_cmd_edit_personal_data (GtkAction *action,
 	g_free (host);
 }
 
-#ifdef ENABLE_CERTIFICATE_MANAGER
-
-#include "ephy-cert-manager-dialog.h"
-
+#if 0
 void
 window_cmd_edit_certificates (GtkAction *action,
 			      EphyWindow *window)
 {
-	EphyDialog *dialog;
-
-	dialog = certs_manager_dialog_new ();
-
-	ephy_dialog_show (dialog);
 }
-
-#endif /* ENABLE_CERTIFICATE_MANAGER */
+#endif
 
 void
 window_cmd_edit_prefs (GtkAction *action,
